/****************************************************************************
** Meta object code from reading C++ file 'mapitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.04.0/src/map-quick/mapitem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mapitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOSMIndoorMap7MapItemE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorMap::MapItem::qt_create_metaobjectdata<qt_meta_tag_ZN13KOSMIndoorMap7MapItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorMap::MapItem",
        "mapDataChanged",
        "",
        "styleSheetChanged",
        "currentFloorLevelChanged",
        "errorChanged",
        "overlaySourcesChanged",
        "regionChanged",
        "timeZoneChanged",
        "hoveredElementChanged",
        "elementAt",
        "KOSMIndoorMap::OSMElement",
        "x",
        "y",
        "loader",
        "KOSMIndoorMap::MapLoader*",
        "view",
        "KOSMIndoorMap::View*",
        "styleSheet",
        "floorLevels",
        "KOSMIndoorMap::FloorLevelModel*",
        "hasError",
        "errorMessage",
        "mapData",
        "KOSMIndoorMap::MapData",
        "overlaySources",
        "QVariant",
        "region",
        "timeZone",
        "hoveredElement"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mapDataChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'styleSheetChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentFloorLevelChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'overlaySourcesChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'regionChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'timeZoneChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hoveredElementChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'elementAt'
        QtMocHelpers::MethodData<KOSMIndoorMap::OSMElement(double, double) const>(10, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::Double, 12 }, { QMetaType::Double, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'loader'
        QtMocHelpers::PropertyData<KOSMIndoorMap::MapLoader*>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'view'
        QtMocHelpers::PropertyData<KOSMIndoorMap::View*>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'styleSheet'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'floorLevels'
        QtMocHelpers::PropertyData<KOSMIndoorMap::FloorLevelModel*>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'hasError'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'errorMessage'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'mapData'
        QtMocHelpers::PropertyData<KOSMIndoorMap::MapData>(23, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'overlaySources'
        QtMocHelpers::PropertyData<QVariant>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'region'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'timeZone'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'hoveredElement'
        QtMocHelpers::PropertyData<KOSMIndoorMap::OSMElement>(29, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MapItem, qt_meta_tag_ZN13KOSMIndoorMap7MapItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSMIndoorMap::MapItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPaintedItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap7MapItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap7MapItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOSMIndoorMap7MapItemE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorMap::MapItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MapItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mapDataChanged(); break;
        case 1: _t->styleSheetChanged(); break;
        case 2: _t->currentFloorLevelChanged(); break;
        case 3: _t->errorChanged(); break;
        case 4: _t->overlaySourcesChanged(); break;
        case 5: _t->regionChanged(); break;
        case 6: _t->timeZoneChanged(); break;
        case 7: _t->hoveredElementChanged(); break;
        case 8: { KOSMIndoorMap::OSMElement _r = _t->elementAt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<KOSMIndoorMap::OSMElement*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MapItem::*)()>(_a, &MapItem::mapDataChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapItem::*)()>(_a, &MapItem::styleSheetChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapItem::*)()>(_a, &MapItem::currentFloorLevelChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapItem::*)()>(_a, &MapItem::errorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapItem::*)()>(_a, &MapItem::overlaySourcesChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapItem::*)()>(_a, &MapItem::regionChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapItem::*)()>(_a, &MapItem::timeZoneChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MapItem::*)()>(_a, &MapItem::hoveredElementChanged, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::FloorLevelModel* >(); break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::MapData >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::MapLoader* >(); break;
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::OSMElement >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::View* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KOSMIndoorMap::MapLoader**>(_v) = _t->loader(); break;
        case 1: *reinterpret_cast<KOSMIndoorMap::View**>(_v) = _t->view(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->styleSheetName(); break;
        case 3: *reinterpret_cast<KOSMIndoorMap::FloorLevelModel**>(_v) = _t->floorLevelModel(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->hasError(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->errorMessage(); break;
        case 6: *reinterpret_cast<KOSMIndoorMap::MapData*>(_v) = _t->mapData(); break;
        case 7: *reinterpret_cast<QVariant*>(_v) = _t->overlaySources(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->region(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->timeZoneId(); break;
        case 10: *reinterpret_cast<KOSMIndoorMap::OSMElement*>(_v) = _t->hoveredElement(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setStylesheetName(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setOverlaySources(*reinterpret_cast<QVariant*>(_v)); break;
        case 8: _t->setRegion(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setTimeZoneId(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setHoveredElement(*reinterpret_cast<KOSMIndoorMap::OSMElement*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KOSMIndoorMap::MapItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KOSMIndoorMap::MapItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap7MapItemE_t>.strings))
        return static_cast<void*>(this);
    return QQuickPaintedItem::qt_metacast(_clname);
}

int KOSMIndoorMap::MapItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPaintedItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KOSMIndoorMap::MapItem::mapDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KOSMIndoorMap::MapItem::styleSheetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KOSMIndoorMap::MapItem::currentFloorLevelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KOSMIndoorMap::MapItem::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KOSMIndoorMap::MapItem::overlaySourcesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KOSMIndoorMap::MapItem::regionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KOSMIndoorMap::MapItem::timeZoneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KOSMIndoorMap::MapItem::hoveredElementChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
