/****************************************************************************
** Meta object code from reading C++ file 'editorcontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.04.0/src/editor/editorcontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'editorcontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4KOSM16EditorControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSM::EditorController::qt_create_metaobjectdata<qt_meta_tag_ZN4KOSM16EditorControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSM::EditorController",
        "hasEditor",
        "",
        "Editor",
        "editor",
        "editElement",
        "OSM::Element",
        "element",
        "editBoundingBox",
        "OSM::BoundingBox",
        "box",
        "ID",
        "JOSM",
        "Vespucci"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'hasEditor'
        QtMocHelpers::MethodData<bool(enum Editor)>(1, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'editElement'
        QtMocHelpers::MethodData<void(OSM::Element, enum Editor)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 3, 4 },
        }}),
        // Method 'editBoundingBox'
        QtMocHelpers::MethodData<void(OSM::BoundingBox, enum Editor)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Editor'
        QtMocHelpers::EnumData<enum Editor>(3, 3, QMC::EnumFlags{}).add({
            {   11, Editor::ID },
            {   12, Editor::JOSM },
            {   13, Editor::Vespucci },
        }),
    };
    return QtMocHelpers::metaObjectData<EditorController, qt_meta_tag_ZN4KOSM16EditorControllerE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSM::EditorController::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KOSM16EditorControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KOSM16EditorControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4KOSM16EditorControllerE_t>.metaTypes,
    nullptr
} };

void KOSM::EditorController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<EditorController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->hasEditor((*reinterpret_cast<std::add_pointer_t<enum Editor>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->editElement((*reinterpret_cast<std::add_pointer_t<OSM::Element>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum Editor>>(_a[2]))); break;
        case 2: _t->editBoundingBox((*reinterpret_cast<std::add_pointer_t<OSM::BoundingBox>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum Editor>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OSM::Element >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< OSM::BoundingBox >(); break;
            }
            break;
        }
    }
}
QT_WARNING_POP
