/****************************************************************************
** Meta object code from reading C++ file 'platformmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.04.0/src/map/content/platformmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'platformmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOSMIndoorMap13PlatformModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorMap::PlatformModel::qt_create_metaobjectdata<qt_meta_tag_ZN13KOSMIndoorMap13PlatformModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorMap::PlatformModel",
        "mapDataChanged",
        "",
        "platformIndexChanged",
        "arrivalPlatformChanged",
        "departurePlatformChanged",
        "setArrivalPlatform",
        "name",
        "KOSMIndoorMap::Platform::Mode",
        "mode",
        "setDeparturePlatform",
        "mapData",
        "KOSMIndoorMap::MapData",
        "isEmpty",
        "arrivalPlatformRow",
        "departurePlatformRow",
        "arrivalPlatform",
        "KOSMIndoorMap::Platform",
        "departurePlatform",
        "Role",
        "CoordinateRole",
        "ElementRole",
        "LevelRole",
        "TransportModeRole",
        "LinesRole",
        "ArrivalPlatformRole",
        "DeparturePlatformRole",
        "PlatformRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mapDataChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'platformIndexChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'arrivalPlatformChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'departurePlatformChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'setArrivalPlatform'
        QtMocHelpers::MethodData<void(const QString &, KOSMIndoorMap::Platform::Mode)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Method 'setDeparturePlatform'
        QtMocHelpers::MethodData<void(const QString &, KOSMIndoorMap::Platform::Mode)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'mapData'
        QtMocHelpers::PropertyData<KOSMIndoorMap::MapData>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'isEmpty'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'arrivalPlatformRow'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'departurePlatformRow'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'arrivalPlatform'
        QtMocHelpers::PropertyData<KOSMIndoorMap::Platform>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'departurePlatform'
        QtMocHelpers::PropertyData<KOSMIndoorMap::Platform>(18, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Role'
        QtMocHelpers::EnumData<enum Role>(19, 19, QMC::EnumFlags{}).add({
            {   20, Role::CoordinateRole },
            {   21, Role::ElementRole },
            {   22, Role::LevelRole },
            {   23, Role::TransportModeRole },
            {   24, Role::LinesRole },
            {   25, Role::ArrivalPlatformRole },
            {   26, Role::DeparturePlatformRole },
            {   27, Role::PlatformRole },
        }),
    };
    return QtMocHelpers::metaObjectData<PlatformModel, qt_meta_tag_ZN13KOSMIndoorMap13PlatformModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSMIndoorMap::PlatformModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap13PlatformModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap13PlatformModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOSMIndoorMap13PlatformModelE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorMap::PlatformModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlatformModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mapDataChanged(); break;
        case 1: _t->platformIndexChanged(); break;
        case 2: _t->arrivalPlatformChanged(); break;
        case 3: _t->departurePlatformChanged(); break;
        case 4: _t->setArrivalPlatform((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KOSMIndoorMap::Platform::Mode>>(_a[2]))); break;
        case 5: _t->setDeparturePlatform((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KOSMIndoorMap::Platform::Mode>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlatformModel::*)()>(_a, &PlatformModel::mapDataChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlatformModel::*)()>(_a, &PlatformModel::platformIndexChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlatformModel::*)()>(_a, &PlatformModel::arrivalPlatformChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlatformModel::*)()>(_a, &PlatformModel::departurePlatformChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::MapData >(); break;
        case 5:
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::Platform >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KOSMIndoorMap::MapData*>(_v) = _t->mapData(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isEmpty(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->arrivalPlatformRow(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->departurePlatformRow(); break;
        case 4: *reinterpret_cast<KOSMIndoorMap::Platform*>(_v) = _t->arrivalPlatform(); break;
        case 5: *reinterpret_cast<KOSMIndoorMap::Platform*>(_v) = _t->departurePlatform(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMapData(*reinterpret_cast<KOSMIndoorMap::MapData*>(_v)); break;
        case 4: _t->setArrivalPlatform(*reinterpret_cast<KOSMIndoorMap::Platform*>(_v)); break;
        case 5: _t->setDeparturePlatform(*reinterpret_cast<KOSMIndoorMap::Platform*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KOSMIndoorMap::PlatformModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KOSMIndoorMap::PlatformModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap13PlatformModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int KOSMIndoorMap::PlatformModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KOSMIndoorMap::PlatformModel::mapDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KOSMIndoorMap::PlatformModel::platformIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KOSMIndoorMap::PlatformModel::arrivalPlatformChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KOSMIndoorMap::PlatformModel::departurePlatformChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
