/****************************************************************************
** Meta object code from reading C++ file 'routingarea.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.04.0/src/routing-quick/routingarea.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'routingarea.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8AreaTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto AreaType::qt_create_metaobjectdata<qt_meta_tag_ZN8AreaTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AreaType",
        "QML.Element",
        "Unwalkable",
        "Stairs",
        "Elevator",
        "Escalator",
        "MovingWalkway",
        "TactilePaving",
        "StreetCrossing",
        "Ramp",
        "Room",
        "Walkable"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AreaType'
        QtMocHelpers::EnumData<AreaType>(0, 0, QMC::EnumFlags{}).add({
            {    2, AreaType::Unwalkable },
            {    3, AreaType::Stairs },
            {    4, AreaType::Elevator },
            {    5, AreaType::Escalator },
            {    6, AreaType::MovingWalkway },
            {    7, AreaType::TactilePaving },
            {    8, AreaType::StreetCrossing },
            {    9, AreaType::Ramp },
            {   10, AreaType::Room },
            {   11, AreaType::Walkable },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    0 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN8AreaTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN8AreaTypeE =
    AreaType::qt_create_metaobjectdata<qt_meta_tag_ZN8AreaTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN8AreaTypeE =
    qt_staticMetaObjectContent_ZN8AreaTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN8AreaTypeE =
    qt_staticMetaObjectContent_ZN8AreaTypeE.relocatingData;

Q_CONSTINIT const QMetaObject AreaType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN8AreaTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN8AreaTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN8AreaTypeE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN8AreaFlagE_t {};
} // unnamed namespace

template <> constexpr inline auto AreaFlag::qt_create_metaobjectdata<qt_meta_tag_ZN8AreaFlagE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AreaFlag",
        "QML.Element",
        "NoFlag",
        "Walkable",
        "Stairs",
        "Escalator",
        "Elevator"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AreaFlag'
        QtMocHelpers::EnumData<AreaFlag>(0, 0, QMC::EnumFlags{}).add({
            {    2, AreaFlag::NoFlag },
            {    3, AreaFlag::Walkable },
            {    4, AreaFlag::Stairs },
            {    5, AreaFlag::Escalator },
            {    6, AreaFlag::Elevator },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    0 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN8AreaFlagE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN8AreaFlagE =
    AreaFlag::qt_create_metaobjectdata<qt_meta_tag_ZN8AreaFlagE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN8AreaFlagE =
    qt_staticMetaObjectContent_ZN8AreaFlagE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN8AreaFlagE =
    qt_staticMetaObjectContent_ZN8AreaFlagE.relocatingData;

Q_CONSTINIT const QMetaObject AreaFlag::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN8AreaFlagE.stringdata,
    qt_staticMetaObjectStaticContent_ZN8AreaFlagE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN8AreaFlagE.metaTypes,
    nullptr
} };

QT_WARNING_POP
