/****************************************************************************
** Meta object code from reading C++ file 'types.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.04.0/src/map-quick/types.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'types.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21EquipmentModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto EquipmentModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN21EquipmentModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EquipmentModelForeign",
        "QML.Element",
        "EquipmentModel",
        "QML.Foreign",
        "KOSMIndoorMap::EquipmentModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<EquipmentModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject EquipmentModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21EquipmentModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21EquipmentModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21EquipmentModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN22FloorLevelModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto FloorLevelModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN22FloorLevelModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FloorLevelModelForeign",
        "QML.Element",
        "FloorLevelModel",
        "QML.Foreign",
        "KOSMIndoorMap::FloorLevelModel",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "only provided via C++ API"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<FloorLevelModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject FloorLevelModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22FloorLevelModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22FloorLevelModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22FloorLevelModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN16GateModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto GateModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN16GateModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GateModelForeign",
        "QML.Element",
        "GateModel",
        "QML.Foreign",
        "KOSMIndoorMap::GateModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<GateModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject GateModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16GateModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16GateModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16GateModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN14MapDataForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto MapDataForeign::qt_create_metaobjectdata<qt_meta_tag_ZN14MapDataForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MapDataForeign",
        "QML.Foreign",
        "KOSMIndoorMap::MapData",
        "QML.Element",
        "mapData",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "only provided via C++ API"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<MapDataForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject MapDataForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MapDataForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MapDataForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14MapDataForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN16MapLoaderForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto MapLoaderForeign::qt_create_metaobjectdata<qt_meta_tag_ZN16MapLoaderForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MapLoaderForeign",
        "QML.Element",
        "MapLoader",
        "QML.Foreign",
        "KOSMIndoorMap::MapLoader",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "only provided via C++ API"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<MapLoaderForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject MapLoaderForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MapLoaderForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MapLoaderForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16MapLoaderForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN15PlatformForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto PlatformForeign::qt_create_metaobjectdata<qt_meta_tag_ZN15PlatformForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlatformForeign",
        "QML.Element",
        "platform",
        "QML.Foreign",
        "KOSMIndoorMap::Platform"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<PlatformForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject PlatformForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlatformForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlatformForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PlatformForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN15PlatformDerivedE_t {};
} // unnamed namespace

template <> constexpr inline auto PlatformDerived::qt_create_metaobjectdata<qt_meta_tag_ZN15PlatformDerivedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlatformDerived"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlatformDerived, qt_meta_tag_ZN15PlatformDerivedE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlatformDerived::staticMetaObject = { {
    QtPrivate::MetaObjectForType<KOSMIndoorMap::Platform>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlatformDerivedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PlatformDerivedE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PlatformDerivedE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN20PlatformModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto PlatformModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN20PlatformModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlatformModelForeign",
        "QML.Element",
        "PlatformModel",
        "QML.Foreign",
        "KOSMIndoorMap::PlatformModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<PlatformModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject PlatformModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20PlatformModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20PlatformModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20PlatformModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN11ViewForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto ViewForeign::qt_create_metaobjectdata<qt_meta_tag_ZN11ViewForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ViewForeign",
        "QML.Element",
        "View",
        "QML.Foreign",
        "KOSMIndoorMap::View",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "only provided via C++ API"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<ViewForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ViewForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ViewForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ViewForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11ViewForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN19AmenityModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto AmenityModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN19AmenityModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AmenityModelForeign",
        "QML.Element",
        "AmenityModel",
        "QML.Foreign",
        "KOSMIndoorMap::AmenityModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<AmenityModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject AmenityModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19AmenityModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19AmenityModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19AmenityModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN34AmenitySortFilterProxyModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto AmenitySortFilterProxyModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN34AmenitySortFilterProxyModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AmenitySortFilterProxyModelForeign",
        "QML.Element",
        "AmenitySortFilterProxyModel",
        "QML.Foreign",
        "KOSMIndoorMap::AmenitySortFilterProxyModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<AmenitySortFilterProxyModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject AmenitySortFilterProxyModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34AmenitySortFilterProxyModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34AmenitySortFilterProxyModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN34AmenitySortFilterProxyModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN28FloorLevelChangeModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto FloorLevelChangeModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN28FloorLevelChangeModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FloorLevelChangeModelForeign",
        "QML.Element",
        "FloorLevelChangeModel",
        "QML.Foreign",
        "KOSMIndoorMap::FloorLevelChangeModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<FloorLevelChangeModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject FloorLevelChangeModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28FloorLevelChangeModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28FloorLevelChangeModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28FloorLevelChangeModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN14MapItemForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto MapItemForeign::qt_create_metaobjectdata<qt_meta_tag_ZN14MapItemForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MapItemForeign",
        "QML.Element",
        "MapItemImpl",
        "QML.Foreign",
        "KOSMIndoorMap::MapItem"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<MapItemForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject MapItemForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MapItemForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MapItemForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14MapItemForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN17OSMElementForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto OSMElementForeign::qt_create_metaobjectdata<qt_meta_tag_ZN17OSMElementForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OSMElementForeign",
        "QML.Element",
        "osmElement",
        "QML.Foreign",
        "KOSMIndoorMap::OSMElement",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "only provided via C++ API"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<OSMElementForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OSMElementForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17OSMElementForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17OSMElementForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17OSMElementForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN33OSMElementInformationModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto OSMElementInformationModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN33OSMElementInformationModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OSMElementInformationModelForeign",
        "QML.Element",
        "OSMElementInformationModel",
        "QML.Foreign",
        "KOSMIndoorMap::OSMElementInformationModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<OSMElementInformationModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OSMElementInformationModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33OSMElementInformationModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33OSMElementInformationModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33OSMElementInformationModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN16RoomModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto RoomModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN16RoomModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RoomModelForeign",
        "QML.Element",
        "RoomModel",
        "QML.Foreign",
        "KOSMIndoorMap::RoomModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<RoomModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject RoomModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16RoomModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16RoomModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16RoomModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN31RoomSortFilterProxyModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto RoomSortFilterProxyModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN31RoomSortFilterProxyModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RoomSortFilterProxyModelForeign",
        "QML.Element",
        "RoomSortFilterProxyModel",
        "QML.Foreign",
        "KOSMIndoorMap::RoomSortFilterProxyModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<RoomSortFilterProxyModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject RoomSortFilterProxyModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31RoomSortFilterProxyModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31RoomSortFilterProxyModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31RoomSortFilterProxyModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN22PlatformDerivedForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto PlatformDerivedForeign::qt_create_metaobjectdata<qt_meta_tag_ZN22PlatformDerivedForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlatformDerivedForeign",
        "QML.Element",
        "Platform",
        "QML.Foreign",
        "PlatformDerived",
        "QML.ForeignIsNamespace",
        "true"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN22PlatformDerivedForeignE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN22PlatformDerivedForeignE =
    PlatformDerivedForeign::qt_create_metaobjectdata<qt_meta_tag_ZN22PlatformDerivedForeignE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN22PlatformDerivedForeignE =
    qt_staticMetaObjectContent_ZN22PlatformDerivedForeignE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN22PlatformDerivedForeignE =
    qt_staticMetaObjectContent_ZN22PlatformDerivedForeignE.relocatingData;

Q_CONSTINIT const QMetaObject PlatformDerivedForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN22PlatformDerivedForeignE.stringdata,
    qt_staticMetaObjectStaticContent_ZN22PlatformDerivedForeignE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN22PlatformDerivedForeignE.metaTypes,
    nullptr
} };

QT_WARNING_POP
