/****************************************************************************
** Meta object code from reading C++ file 'routingprofile.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.04.0/src/routing/routingprofile.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'routingprofile.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17KOSMIndoorRouting14RoutingProfileE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorRouting::RoutingProfile::qt_create_metaobjectdata<qt_meta_tag_ZN17KOSMIndoorRouting14RoutingProfileE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorRouting::RoutingProfile",
        "cost",
        "",
        "KOSMIndoorRouting::AreaType",
        "area",
        "setCost",
        "flags",
        "KOSMIndoorRouting::AreaFlags"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'cost'
        QtMocHelpers::MethodData<float(KOSMIndoorRouting::AreaType) const>(1, 2, QMC::AccessPublic, QMetaType::Float, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'setCost'
        QtMocHelpers::MethodData<void(KOSMIndoorRouting::AreaType, float)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Float, 1 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'flags'
        QtMocHelpers::PropertyData<KOSMIndoorRouting::AreaFlags>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RoutingProfile, qt_meta_tag_ZN17KOSMIndoorRouting14RoutingProfileE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSMIndoorRouting::RoutingProfile::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KOSMIndoorRouting14RoutingProfileE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17KOSMIndoorRouting14RoutingProfileE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17KOSMIndoorRouting14RoutingProfileE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorRouting::RoutingProfile::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<RoutingProfile *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { float _r = _t->cost((*reinterpret_cast<std::add_pointer_t<KOSMIndoorRouting::AreaType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->setCost((*reinterpret_cast<std::add_pointer_t<KOSMIndoorRouting::AreaType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KOSMIndoorRouting::AreaFlags*>(_v) = _t->flags(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlags(*reinterpret_cast<KOSMIndoorRouting::AreaFlags*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
