/****************************************************************************
** Meta object code from reading C++ file 'view.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.04.0/src/map/scene/view.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'view.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOSMIndoorMap4ViewE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorMap::View::qt_create_metaobjectdata<qt_meta_tag_ZN13KOSMIndoorMap4ViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorMap::View",
        "transformationChanged",
        "",
        "floorLevelChanged",
        "timeChanged",
        "mapSceneToGeo",
        "OSM::BoundingBox",
        "QRectF",
        "box",
        "setZoomLevel",
        "zoom",
        "QPointF",
        "screenCenter",
        "mapScreenToScene",
        "screenPos",
        "mapMetersToScreen",
        "meters",
        "mapScreenToMeters",
        "pixels",
        "zoomIn",
        "zoomOut",
        "panTopLeft",
        "x",
        "y",
        "centerOnGeoCoordinate",
        "geoCoord",
        "centerOn",
        "floorLevel",
        "zoomLevel",
        "mapSceneToGeoPoint",
        "p",
        "panX",
        "panY",
        "panWidth",
        "panHeight",
        "viewport",
        "beginTime",
        "endTime"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'transformationChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'floorLevelChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'timeChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'mapSceneToGeo'
        QtMocHelpers::MethodData<OSM::BoundingBox(const QRectF &)>(5, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'setZoomLevel'
        QtMocHelpers::MethodData<void(double, QPointF)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Method 'mapScreenToScene'
        QtMocHelpers::MethodData<QPointF(QPointF) const>(13, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 11, 14 },
        }}),
        // Method 'mapMetersToScreen'
        QtMocHelpers::MethodData<double(double) const>(15, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::Double, 16 },
        }}),
        // Method 'mapScreenToMeters'
        QtMocHelpers::MethodData<double(int) const>(17, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::Int, 18 },
        }}),
        // Method 'zoomIn'
        QtMocHelpers::MethodData<void(QPointF)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Method 'zoomOut'
        QtMocHelpers::MethodData<void(QPointF)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Method 'panTopLeft'
        QtMocHelpers::MethodData<void(double, double)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 22 }, { QMetaType::Double, 23 },
        }}),
        // Method 'centerOnGeoCoordinate'
        QtMocHelpers::MethodData<void(QPointF)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 25 },
        }}),
        // Method 'centerOn'
        QtMocHelpers::MethodData<void(QPointF, int, double)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 25 }, { QMetaType::Int, 27 }, { QMetaType::Double, 28 },
        }}),
        // Method 'mapSceneToGeoPoint'
        QtMocHelpers::MethodData<QPointF(QPointF)>(29, 2, QMC::AccessProtected, 0x80000000 | 11, {{
            { 0x80000000 | 11, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'panX'
        QtMocHelpers::PropertyData<double>(31, QMetaType::Double, QMC::DefaultPropertyFlags, 0),
        // property 'panY'
        QtMocHelpers::PropertyData<double>(32, QMetaType::Double, QMC::DefaultPropertyFlags, 0),
        // property 'panWidth'
        QtMocHelpers::PropertyData<double>(33, QMetaType::Double, QMC::DefaultPropertyFlags, 0),
        // property 'panHeight'
        QtMocHelpers::PropertyData<double>(34, QMetaType::Double, QMC::DefaultPropertyFlags, 0),
        // property 'floorLevel'
        QtMocHelpers::PropertyData<int>(27, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'viewport'
        QtMocHelpers::PropertyData<QRectF>(35, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'zoomLevel'
        QtMocHelpers::PropertyData<double>(28, QMetaType::Double, QMC::DefaultPropertyFlags, 0),
        // property 'beginTime'
        QtMocHelpers::PropertyData<QDateTime>(36, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'endTime'
        QtMocHelpers::PropertyData<QDateTime>(37, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<View, qt_meta_tag_ZN13KOSMIndoorMap4ViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSMIndoorMap::View::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap4ViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap4ViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOSMIndoorMap4ViewE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorMap::View::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<View *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->transformationChanged(); break;
        case 1: _t->floorLevelChanged(); break;
        case 2: _t->timeChanged(); break;
        case 3: { OSM::BoundingBox _r = _t->mapSceneToGeo((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<OSM::BoundingBox*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->setZoomLevel((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 5: { QPointF _r = _t->mapScreenToScene((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        case 6: { double _r = _t->mapMetersToScreen((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 7: { double _r = _t->mapScreenToMeters((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->zoomIn((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 9: _t->zoomOut((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 10: _t->panTopLeft((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 11: _t->centerOnGeoCoordinate((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 12: _t->centerOn((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 13: { QPointF _r = _t->mapSceneToGeoPoint((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (View::*)()>(_a, &View::transformationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (View::*)()>(_a, &View::floorLevelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (View::*)()>(_a, &View::timeChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->panX(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->panY(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->panWidth(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->panHeight(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->level(); break;
        case 5: *reinterpret_cast<QRectF*>(_v) = _t->viewport(); break;
        case 6: *reinterpret_cast<double*>(_v) = _t->zoomLevel(); break;
        case 7: *reinterpret_cast<QDateTime*>(_v) = _t->beginTime(); break;
        case 8: *reinterpret_cast<QDateTime*>(_v) = _t->endTime(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 4: _t->setLevel(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setBeginTime(*reinterpret_cast<QDateTime*>(_v)); break;
        case 8: _t->setEndTime(*reinterpret_cast<QDateTime*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KOSMIndoorMap::View::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KOSMIndoorMap::View::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap4ViewE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KOSMIndoorMap::View::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KOSMIndoorMap::View::transformationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KOSMIndoorMap::View::floorLevelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KOSMIndoorMap::View::timeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
