/****************************************************************************
** Meta object code from reading C++ file 'osmelementinformationmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.04.0/src/map-quick/osmelementinformationmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'osmelementinformationmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOSMIndoorMap26OSMElementInformationModelE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorMap::OSMElementInformationModel::qt_create_metaobjectdata<qt_meta_tag_ZN13KOSMIndoorMap26OSMElementInformationModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorMap::OSMElementInformationModel",
        "elementChanged",
        "",
        "allowOnlineContentChanged",
        "debugChanged",
        "clear",
        "element",
        "KOSMIndoorMap::OSMElement",
        "name",
        "category",
        "allowOnlineContent",
        "debug",
        "KeyCategory",
        "UnresolvedCategory",
        "Header",
        "Main",
        "OpeningHoursCategory",
        "Contact",
        "Payment",
        "Toilets",
        "Accessibility",
        "Parking",
        "Operator",
        "DebugCategory",
        "Key",
        "NoKey",
        "Name",
        "Category",
        "Image",
        "Logo",
        "OldName",
        "Description",
        "Routes",
        "Cuisine",
        "VendingMachineOffer",
        "Diet",
        "Takeaway",
        "Socket",
        "OpeningHours",
        "AvailableVehicles",
        "Fee",
        "Authentication",
        "BicycleParking",
        "Capacity",
        "CapacityDisabled",
        "CapacityWomen",
        "CapacityParent",
        "CapacityCharing",
        "MaxStay",
        "DiaperChangingTable",
        "Gender",
        "Wikipedia",
        "Address",
        "Phone",
        "Email",
        "Website",
        "PaymentCash",
        "PaymentDigital",
        "PaymentDebitCard",
        "PaymentCreditCard",
        "PaymentStoredValueCard",
        "Wheelchair",
        "WheelchairLift",
        "CentralKey",
        "SpeechOutput",
        "TactileWriting",
        "OperatorName",
        "Network",
        "OperatorWikipedia",
        "RemainingRange",
        "DebugLink",
        "DebugKey",
        "Type",
        "String",
        "Link",
        "PostalAddress",
        "OpeningHoursType",
        "ImageType"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'elementChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'allowOnlineContentChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'debugChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'element'
        QtMocHelpers::PropertyData<KOSMIndoorMap::OSMElement>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'category'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'allowOnlineContent'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'debug'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'KeyCategory'
        QtMocHelpers::EnumData<enum KeyCategory>(12, 12, QMC::EnumFlags{}).add({
            {   13, KeyCategory::UnresolvedCategory },
            {   14, KeyCategory::Header },
            {   15, KeyCategory::Main },
            {   16, KeyCategory::OpeningHoursCategory },
            {   17, KeyCategory::Contact },
            {   18, KeyCategory::Payment },
            {   19, KeyCategory::Toilets },
            {   20, KeyCategory::Accessibility },
            {   21, KeyCategory::Parking },
            {   22, KeyCategory::Operator },
            {   23, KeyCategory::DebugCategory },
        }),
        // enum 'Key'
        QtMocHelpers::EnumData<enum Key>(24, 24, QMC::EnumFlags{}).add({
            {   25, Key::NoKey },
            {   26, Key::Name },
            {   27, Key::Category },
            {   28, Key::Image },
            {   29, Key::Logo },
            {   30, Key::OldName },
            {   31, Key::Description },
            {   32, Key::Routes },
            {   33, Key::Cuisine },
            {   34, Key::VendingMachineOffer },
            {   35, Key::Diet },
            {   36, Key::Takeaway },
            {   37, Key::Socket },
            {   38, Key::OpeningHours },
            {   39, Key::AvailableVehicles },
            {   40, Key::Fee },
            {   41, Key::Authentication },
            {   42, Key::BicycleParking },
            {   43, Key::Capacity },
            {   44, Key::CapacityDisabled },
            {   45, Key::CapacityWomen },
            {   46, Key::CapacityParent },
            {   47, Key::CapacityCharing },
            {   48, Key::MaxStay },
            {   49, Key::DiaperChangingTable },
            {   50, Key::Gender },
            {   51, Key::Wikipedia },
            {   52, Key::Address },
            {   53, Key::Phone },
            {   54, Key::Email },
            {   55, Key::Website },
            {   56, Key::PaymentCash },
            {   57, Key::PaymentDigital },
            {   58, Key::PaymentDebitCard },
            {   59, Key::PaymentCreditCard },
            {   60, Key::PaymentStoredValueCard },
            {   61, Key::Wheelchair },
            {   62, Key::WheelchairLift },
            {   63, Key::CentralKey },
            {   64, Key::SpeechOutput },
            {   65, Key::TactileWriting },
            {   66, Key::OperatorName },
            {   67, Key::Network },
            {   68, Key::OperatorWikipedia },
            {   69, Key::RemainingRange },
            {   70, Key::DebugLink },
            {   71, Key::DebugKey },
        }),
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(72, 72, QMC::EnumFlags{}).add({
            {   73, Type::String },
            {   74, Type::Link },
            {   75, Type::PostalAddress },
            {   76, Type::OpeningHoursType },
            {   77, Type::ImageType },
        }),
    };
    return QtMocHelpers::metaObjectData<OSMElementInformationModel, qt_meta_tag_ZN13KOSMIndoorMap26OSMElementInformationModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSMIndoorMap::OSMElementInformationModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap26OSMElementInformationModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap26OSMElementInformationModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOSMIndoorMap26OSMElementInformationModelE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorMap::OSMElementInformationModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OSMElementInformationModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->elementChanged(); break;
        case 1: _t->allowOnlineContentChanged(); break;
        case 2: _t->debugChanged(); break;
        case 3: _t->clear(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OSMElementInformationModel::*)()>(_a, &OSMElementInformationModel::elementChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OSMElementInformationModel::*)()>(_a, &OSMElementInformationModel::allowOnlineContentChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OSMElementInformationModel::*)()>(_a, &OSMElementInformationModel::debugChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KOSMIndoorMap::OSMElement >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KOSMIndoorMap::OSMElement*>(_v) = _t->element(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->category(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->m_allowOnlineContent; break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->m_debug; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setElement(*reinterpret_cast<KOSMIndoorMap::OSMElement*>(_v)); break;
        case 3:
            if (QtMocHelpers::setProperty(_t->m_allowOnlineContent, *reinterpret_cast<bool*>(_v)))
                Q_EMIT _t->allowOnlineContentChanged();
            break;
        case 4:
            if (QtMocHelpers::setProperty(_t->m_debug, *reinterpret_cast<bool*>(_v)))
                Q_EMIT _t->debugChanged();
            break;
        default: break;
        }
    }
}

const QMetaObject *KOSMIndoorMap::OSMElementInformationModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KOSMIndoorMap::OSMElementInformationModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap26OSMElementInformationModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int KOSMIndoorMap::OSMElementInformationModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KOSMIndoorMap::OSMElementInformationModel::elementChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KOSMIndoorMap::OSMElementInformationModel::allowOnlineContentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KOSMIndoorMap::OSMElementInformationModel::debugChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
