/****************************************************************************
** Meta object code from reading C++ file 'osmaddress.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kosmindoormap-26.04.0/src/map-quick/osmaddress.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'osmaddress.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KOSMIndoorMap10OSMAddressE_t {};
} // unnamed namespace

template <> constexpr inline auto KOSMIndoorMap::OSMAddress::qt_create_metaobjectdata<qt_meta_tag_ZN13KOSMIndoorMap10OSMAddressE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KOSMIndoorMap::OSMAddress",
        "street",
        "houseNumber",
        "postalCode",
        "city",
        "state",
        "country"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'street'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'houseNumber'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'postalCode'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'city'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'state'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'country'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OSMAddress, qt_meta_tag_ZN13KOSMIndoorMap10OSMAddressE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KOSMIndoorMap::OSMAddress::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap10OSMAddressE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KOSMIndoorMap10OSMAddressE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KOSMIndoorMap10OSMAddressE_t>.metaTypes,
    nullptr
} };

void KOSMIndoorMap::OSMAddress::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<OSMAddress *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->street(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->houseNumber(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->postalCode(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->city(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->state(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->country(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
